USE mssqltips
GO

SELECT *
INTO dbo.imp_SalesOrderHeader
FROM AdventureWorks.Sales.SalesOrderHeader

SELECT *
INTO dbo.imp_SalesOrderDetail
FROM AdventureWorks.Sales.SalesOrderDetail

ALTER TABLE dbo.imp_SalesOrderHeader
ADD Processed bit not null default 0
GO


-- create stored procedure: stp_CreateOrderBatchList
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE dbo.stp_CreateOrderBatchList 
AS
BEGIN
	SET NOCOUNT ON;

	SELECT 
     DATEPART(YYYY,OrderDate) OrderYear
    ,DATEPART(MONTH,OrderDate) OrderMonth
	FROM dbo.imp_SalesOrderHeader
	WHERE Processed = 0
	GROUP BY 
     DATEPART(YYYY,OrderDate)
	,DATEPART(MONTH,OrderDate)
	ORDER BY 
     DATEPART(YYYY,OrderDate)
	,DATEPART(MONTH,OrderDate)
END
GO


-- create sales history table
CREATE TABLE dbo.SalesHistory (
OrderYear int not null,
OrderMonth int not null,
ProductID int not null,
OrderQty smallint not null,
LineTotal money not null
)


-- create stored procedure: stp_AppendSalesHistory
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE dbo.stp_AppendSalesHistory 
 @OrderYear int
,@OrderMonth int
AS
BEGIN
	SET NOCOUNT ON;

	INSERT INTO dbo.SalesHistory (
	 OrderYear
	,OrderMonth
	,ProductID
	,OrderQty
	,LineTotal 
	)
	SELECT
     DATEPART(YYYY,m.OrderDate)
    ,DATEPART(MONTH,m.OrderDate)
	,d.ProductID
	,d.OrderQty
	,d.LineTotal
	FROM dbo.imp_SalesOrderHeader m
	JOIN dbo.imp_SalesOrderDetail d ON d.SalesOrderID = m.SalesOrderID
	WHERE Processed = 0 
    AND DATEPART(YYYY,m.OrderDate) = @OrderYear
	AND DATEPART(MONTH,m.OrderDate) = @OrderMonth
END
GO


-- create sales history summary table
CREATE TABLE dbo.SalesHistorySummary (
OrderYear int not null,
OrderMonth int not null,
ProductID int not null,
OrderQty smallint not null,
LineTotal money not null
)


-- create stored procedure: stp_CalcSalesHistorySummary
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE dbo.stp_CalcSalesHistorySummary 
 @OrderYear int
,@OrderMonth int
AS
BEGIN
	SET NOCOUNT ON;

	DELETE FROM dbo.SalesHistorySummary
	WHERE OrderYear = @OrderYear
	AND OrderMonth = @OrderMonth;

	INSERT INTO dbo.SalesHistorySummary (
	 OrderYear
	,OrderMonth
	,ProductID
	,OrderQty
	,LineTotal 
	)
	SELECT
     OrderYear
    ,OrderMonth
	,ProductID
	,SUM(OrderQty)
	,SUM(LineTotal)
	FROM dbo.SalesHistory
	WHERE OrderYear = @OrderYear
	AND OrderMonth = @OrderMonth
	GROUP BY 
     OrderYear
	,OrderMonth
	,ProductID
END
GO


-- create stored procedure: stp_MarkOrdersProcessed
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE dbo.stp_MarkOrdersProcessed 
 @OrderYear int
,@OrderMonth int
AS
BEGIN
	SET NOCOUNT ON;

	UPDATE dbo.imp_SalesOrderHeader
	SET Processed = 1
	WHERE DATEPART(YYYY,OrderDate) = @OrderYear
	AND DATEPART(MONTH,OrderDate) = @OrderMonth;
END
GO

